import React, { createContext, useContext, useEffect, useState } from 'react';
import { Appearance, type ColorSchemeName } from 'react-native';
import type { ThemeContextProps, ThemeProviderProps } from './types';
import { CUSTOM_THEME, DARK_THEME, LIGHT_THEME } from './color';
import type { Theme } from './color/types';

const ThemeContext = createContext<ThemeContextProps | undefined>(undefined);

export const ThemeProvider: React.FC<ThemeProviderProps> = ({
  children,
  themeConfig = {},
  useTheme = 'light',
}) => {
  const [theme, setTheme] = useState<ColorSchemeName | 'custom'>(
    useTheme || Appearance.getColorScheme() || 'light'
  );

  useEffect(() => {
    const subscription = Appearance.addChangeListener(({ colorScheme }) => {
      setTheme(colorScheme || 'light');
    });

    return () => {
      subscription.remove();
    };
  }, []);

  const toggleTheme = () => {
    setTheme((prevTheme) => {
      if (prevTheme === 'light') {
        return 'dark';
      } else if (prevTheme === 'dark') {
        return 'light';
      } else {
        return 'light';
      }
    });
  };

  const getColorsForTheme = () => {
    const themeColors =
      theme === 'light'
        ? {
            ...LIGHT_THEME,
            ...themeConfig.light,
          }
        : theme === 'dark'
          ? {
              ...DARK_THEME,
              ...themeConfig.dark,
            }
          : {
              ...CUSTOM_THEME,
              ...themeConfig.custom,
            };

    return themeColors;
  };

  const colors: Theme = getColorsForTheme();

  return (
    <ThemeContext.Provider
      value={
        {
          theme: theme as ColorSchemeName,
          toggleTheme,
          colors,
        } as ThemeContextProps
      }
    >
      {children}
    </ThemeContext.Provider>
  );
};

export const useTheme = () => {
  const context = useContext(ThemeContext);
  if (!context) {
    throw new Error('useTheme must be used within a ThemeProvider');
  }
  return context;
};
